@@ -27,11 +27,14 @@ class ModelsEvaluator:
     def __init__(self):
         self.featuresHandler = FeaturesHandler()
 
-    def evaluate_model(self,model, X_eval, y_eval):
+    def evaluate_model(self,model, eval_data, text_data):
         """Evaluate the model on the held-out evaluation data
 
         Return the f-value for disaster-related and the AUC
         """
+        
+        X_eval, y_eval, ids = eval_data
+
         related_confs = [] # related items and their confidence of being related
         not_related_confs = [] # not related items and their confidence of being _related_
 
@@ -43,32 +46,38 @@ class ModelsEvaluator:
         false_neg = 0.0
         true_neg = 0.0
         
+        true_pos_text = []
+        false_neg_text = []
+        false_pos_text = []
+        true_neg_text = []
+
         with torch.no_grad():
             print("torch.no_grad() go:")
-            for feature_vector, label in zip(X_eval, y_eval):
+            for feature_vector, label, _id in zip(X_eval, y_eval, ids):
 
                 log_probs = model(feature_vector)
                 # get confidence that item is disaster-related
                 prob_related = math.exp(log_probs.data.tolist()[0][1])
 
-                print("type(label) : ", type(label))
+                #print("type(label) : ", type(label))
                 if ((type(label) == str) & (label == "1")) |  ((type(label) == int) & (label == 1)):
                     # true label is disaster related
                     related_confs.append(prob_related)
                     if prob_related > 0.5:
                         true_pos += 1.0
-                        #print("true_pos text : ", text)
+                        true_pos_text.append(text_data[_id][1])
                     else:
                         false_neg += 1.0
-                        #print("false_neg text : ", text)
+                        false_neg_text.append(text_data[_id][1])
                 else:
                     # not disaster-related
                     not_related_confs.append(prob_related)
                     if prob_related > 0.5:
                         false_pos += 1.0
-                        #print("false_neg text : ", text)
+                        false_pos_text.append(text_data[_id][1])
                     else:
                         true_neg += 1.0
+                        true_neg_text.append(text_data[_id][1])
 
         # Get FScore
         if true_pos == 0.0:
@@ -98,7 +107,7 @@ class ModelsEvaluator:
         else:
             auc = total_greater / denom
 
-        return[fscore, auc, precision, recall, true_pos, false_pos, false_neg, true_neg]
+        return[fscore, auc, precision, recall, true_pos, false_pos, false_neg, true_neg, true_pos_text, false_neg_text, false_pos_text, true_neg_text]
 
 class SGDEngine:
     def __init__(self):
@@ -118,10 +127,12 @@ class SGDEngine:
     def get_metrics_by_batch(self):
         return self.fscores, self.aucs, self.precisions, self.recalls
 
-    def process(self, model, X_train, y_train, epochs, select_per_epoch):
+    def process(self, model, eval_data, text_data, epochs, select_per_epoch):
         if model == None:
             raise Exception("model cannot be None")
 
+        X_train, y_train, ids_train = eval_data
+
         self.model = model
         loss_function = nn.NLLLoss()
         optimizer = optim.SGD(self.model.parameters(), lr=0.01)
@@ -134,6 +145,7 @@ class SGDEngine:
             self.model.zero_grad() 
             epoch_X_train = X_train[:select_per_epoch]
             epoch_y_train = y_train[:select_per_epoch]
+            epoch_ids_train = ids_train[:select_per_epoch]
             for item in zip(epoch_X_train, epoch_y_train):
                 x, y = item
                 target = torch.LongTensor([int(y)]) # pylint: disable=no-member
@@ -144,7 +156,8 @@ class SGDEngine:
                 loss.backward()
                 optimizer.step()
 
-            fscore, auc, precision, recall, _, _, _, _ = self.modelsEvaluator.evaluate_model(self.model, epoch_X_train, epoch_y_train)
+            train_data = epoch_X_train,epoch_y_train,epoch_ids_train
+            fscore, auc, precision, recall, _, _, _, _, _, _, _, _ = self.modelsEvaluator.evaluate_model(self.model, train_data,text_data)
             self.fscores.append(fscore)
             self.aucs.append(auc)
             self.precisions.append(precision)
@@ -156,8 +169,8 @@ class EvalModelPloter:
     def __init__(self, run):
         self.run = run
         self.modelsEvaluator = ModelsEvaluator()
-    
-    def evaluate(self, model, X_eval, y_eval):
+
+    def evaluate(self, model, eval_data, text_data):
         """retrieve model metrics and log them in the Workspace
 
         Arguments:
@@ -170,6 +183,7 @@ class EvalModelPloter:
             Exception: data cannot be empty
             Exception: feature_index cannot be empty
         """
+        #X_eval, y_eval,ids = eval_data
 
         if not model :
             raise Exception("model cannot be empty") 
@@ -181,7 +195,8 @@ class EvalModelPloter:
             raise Exception("feature_index cannot be empty") 
 
         # Evaluation of trained model thanks to evaluation data (evaluation_data)
-        fscore, auc, precision, recall, true_pos, false_pos, false_neg, true_neg = self.modelsEvaluator.evaluate_model(model,X_eval,y_eval)
+        result = self.modelsEvaluator.evaluate_model(model, eval_data, text_data)
+        fscore, auc, precision, recall, true_pos, false_pos, false_neg, true_neg,true_pos_text, false_neg_text, false_pos_text, true_neg_text = result
         # log fscore metric to azure experiment
         self.run.log("fscore", round(fscore,5))
         # log auc metric to azure experiment
@@ -191,11 +206,18 @@ class EvalModelPloter:
         #log recall metric to azure experiment
         self.run.log("recall", round(recall,5)) 
 
+
+        self.run.log_list("true_pos_text",true_pos_text)
+        self.run.log_list("false_neg_text",false_neg_text)
+        self.run.log_list("false_pos_text",false_pos_text)
+        self.run.log_list("true_neg_text",true_neg_text)
+
         #plot cofusion matrix
         cm = [[int(true_pos), int(false_pos)], [int(false_neg), int(true_neg)]]
         cm_plot = PlotUtilities._get_confusion_matrix_plot(cm)
         self.run.log_image("Confusion Matrix", plot = cm_plot)
 
+
         return fscore, auc, precision, recall
 
 
